@@ -46,13 +46,14 @@ def create_parser():
     run.add_argument('--ncpu', help='Number of parallel processes', type=int, default=os.cpu_count())
     run.add_argument('--dime', help='Specify DiME streaming server address and port', type=str)
     run.add_argument('--tf', help='End time of time-domain simulation', type=float)
-    run.add_argument('--convert', help='Convert to format.', type=str, default='', nargs='?')
+    run.add_argument('-c', '--convert', help='Convert to format.', type=str, default='', nargs='?')
+    run.add_argument('-b', '--add-book', help='Add a template workbook for the specified model.', type=str)
     run.add_argument('--convert-all', help='Convert to format with all templates.', type=str, default='',
                      nargs='?')
-    run.add_argument('--add-book', help='Add a template workbook for the specified model.', type=str)
     run.add_argument('--state-matrix', help='Export state matrix to a .mat file. Need to run with `-r eig`',
                      action='store_true')
     run.add_argument('--profile', action='store_true', help='Enable Python cProfiler')
+    run.add_argument('--shell', action='store_true', help='Start in IPython shell')
 
     plot = sub_parsers.add_parser('plot')
     plot.add_argument('filename', nargs=1, default=[], help='simulation output file name, which should end '
@@ -77,13 +78,24 @@ def create_parser():
     plot.add_argument('-f', '--format', dest='save_format',
                       help='format for savefig. Common formats such as png, pdf, jpg are supported')
     plot.add_argument('--dpi', type=int, help='image resolution in dot per inch (DPI)')
+    plot.add_argument('--font-size', type=float, help='Text font size', default=12)
+    plot.add_argument('--line-width', type=float, help='Plot line width', default=1.5)
     plot.add_argument('-g', '--grid', action='store_true', help='grid on')
+    plot.add_argument('-t', '--title', help='title text')
     plot.add_argument('--greyscale', action='store_true', help='greyscale on')
     plot.add_argument('-d', '--no-latex', action='store_false', dest='latex', help='disable LaTex formatting')
     plot.add_argument('-n', '--no-show', action='store_false', dest='show', help='do not show the plot window')
     plot.add_argument('--ytimes', type=str, help='scale the y-axis values by YTIMES')
     plot.add_argument('-c', '--tocsv', help='convert npy output to csv', action='store_true')
 
+    doc = sub_parsers.add_parser('doc')  # NOQA
+    doc.add_argument('attribute', help='System attribute name to get documentation', nargs='?')
+    doc.add_argument('--config', '-c', help='Config help')
+    doc.add_argument('--list', '-l', help='List supported models and groups', action='store_true',
+                     dest='list_supported')
+    doc.add_argument('--init-seq', help='Show model initialization sequence', action='store_true',
+                     )
+
     misc = sub_parsers.add_parser('misc')
     config_exclusive = misc.add_mutually_exclusive_group()
     config_exclusive.add_argument('--edit-config', help='Quick edit of the config file',
@@ -92,15 +104,13 @@ def create_parser():
                                   nargs='?', type=str, default='')
     misc.add_argument('--license', action='store_true', help='Display software license', dest='show_license')
     misc.add_argument('-C', '--clean', help='Clean output files', action='store_true')
+    misc.add_argument('-r', '--recursive', help='Recursively clean outputs (combined useage with --clean)',
+                      action='store_true')
 
     prep = sub_parsers.add_parser('prepare')  # NOQA
     prep.add_argument('-q', '--quick', action='store_true', help='quick processing by skipping pretty prints')
-
-    doc = sub_parsers.add_parser('doc')  # NOQA
-    doc.add_argument('attribute', help='System attribute name to get documentation', nargs='?')
-    doc.add_argument('--config', '-c', help='Config help')
-    doc.add_argument('--list', '-l', help='List supported models and groups', action='store_true',
-                     dest='list_supported')
+    prep.add_argument('-i', '--incremental', action='store_true', help='Only incrementally generate for updated '
+                                                                       'models')
 
     selftest = sub_parsers.add_parser('selftest')  # NOQA
 
@@ -119,8 +129,8 @@ def preamble():
     logger.info("\n"
                 rf"    _           _         | Version {version}" + '\n'
                 rf"   /_\  _ _  __| |___ ___ | Python {py_version} on {system_name}, {date_time}" + '\n'
-                rf"  / _ \| ' \/ _` / -_|_-< | " + "\n"
-                rf' /_/ \_\_||_\__,_\___/__/ | This program comes with ABSOLUTELY NO WARRANTY.' + '\n')
+                r"  / _ \| ' \/ _` / -_|_-< | " + "\n"
+                r' /_/ \_\_||_\__,_\___/__/ | This program comes with ABSOLUTELY NO WARRANTY.' + '\n')
 
     log_path = find_log_path(logging.getLogger("andes"))
     if len(log_path):
@@ -141,7 +151,7 @@ def main():
 
     module = importlib.import_module('andes.main')
 
-    if args.command != 'plot':
+    if args.command not in ('plot', 'doc'):
         preamble()
 
     if args.command is None:
@@ -149,4 +159,4 @@ def main():
 
     else:
         func = getattr(module, args.command)
-        func(**vars(args))
+        return func(cli=True, **vars(args))
