@@ -1,6 +1,7 @@
 import logging
 from typing import Optional, Union, Tuple, List
 from andes.shared import np
+from andes.utils.tab import Tab
 from andes.utils.func import interp_n2
 
 logger = logging.getLogger(__name__)
@@ -22,6 +23,7 @@ class Discrete(object):
         self.export_flags_tex = []
         self.x_set = list()
         self.y_set = list()  # NOT being used
+        self.warn_flags = []  # flags in `warn_flags` not initialized to zero will be warned
 
     def check_var(self, *args, **kwargs):
         """
@@ -33,7 +35,7 @@ class Discrete(object):
 
     def check_eq(self):
         """
-        This function is called in ``l_update_eq`` after updating equations.
+        This function is called in ``l_check_eq`` after updating equations.
 
         It should update internal flags only.
         """
@@ -58,6 +60,23 @@ class Discrete(object):
         return [f'{self.name}_{flag}' for flag in self.export_flags]
 
     def get_tex_names(self):
+        """
+        Return tex_names of exported flags.
+
+        TODO: Fix the bug described in the warning below.
+
+        Warnings
+        --------
+        If underscore `_` appears in both flag tex_name and `self.tex_name` (for example, when this discrete is
+        within a block), the exported tex_name will become invalid for SymPy.
+        Variable name substitution will fail.
+
+        Returns
+        -------
+        list
+            A list of tex_names for all exported flags.
+        """
+
         return [rf'{flag_tex}^{self.tex_name}' for flag_tex in self.export_flags_tex]
 
     def get_values(self):
@@ -69,7 +88,32 @@ class Discrete(object):
 
     def list2array(self, n):
         for flag in self.export_flags:
-            self.__dict__[flag] = self.__dict__[flag] * np.ones(n)
+            self.__dict__[flag] = self.__dict__[flag] * np.ones(n, dtype=float)
+
+    def warn_init_limit(self):
+        """
+        Warn if initialized at limits.
+        """
+        for f, limit in self.warn_flags:
+            if f not in self.export_flags:
+                logger.error(f'warn_flags contain unknown flag {f}')
+                continue
+
+            pos = np.argwhere(np.not_equal(self.__dict__[f], 0)).ravel()
+            if not len(pos):
+                continue
+            err_msg = f'{self.owner.class_name}.{self.name} {self.__dict__[limit].name} at limits'
+            err_data = {'idx': [self.owner.idx.v[i] for i in pos],
+                        'Flag': [f] * len(pos),
+                        'Input Value': self.u.v[pos],
+                        'Limit': self.__dict__[limit].v[pos]
+                        }
+
+            tab = Tab(title=err_msg,
+                      header=err_data.keys(),
+                      data=list(map(list, zip(*err_data.values()))))
+
+            logger.warning(tab.draw())
 
 
 class LessThan(Discrete):
@@ -136,6 +180,10 @@ class Limiter(Discrete):
         Parameter instance for the lower limit
     upper : BaseParam
         Parameter instance for the upper limit
+    no_lower : bool
+        True to only use the upper limit
+    no_upper : bool
+        True to only use the lower limit
 
     Attributes
     ----------
@@ -148,19 +196,31 @@ class Limiter(Discrete):
         Flags for violating the upper limit
     """
 
-    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None):
+    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None,
+                 no_upper=False, no_lower=False):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = u
         self.lower = lower
         self.upper = upper
         self.enable = enable
+        self.no_upper = no_upper
+        self.no_lower = no_lower
 
         self.zu = np.array([0])
         self.zl = np.array([0])
         self.zi = np.array([1])
 
-        self.export_flags = ['zl', 'zi', 'zu']
-        self.export_flags_tex = ['z_l', 'z_i', 'z_u']
+        self.export_flags = ['zi']
+        self.export_flags_tex = ['z_i']
+
+        if not self.no_lower:
+            self.export_flags.append('zl')
+            self.export_flags_tex.append('z_l')
+            self.warn_flags.append(('zl', 'lower'))
+        if not self.no_upper:
+            self.export_flags.append('zu')
+            self.export_flags_tex.append('z_u')
+            self.warn_flags.append(('zu', 'upper'))
 
     def check_var(self, *args, **kwargs):
         """
@@ -168,8 +228,12 @@ class Limiter(Discrete):
         """
         if not self.enable:
             return
-        self.zu[:] = np.greater_equal(self.u.v, self.upper.v)
-        self.zl[:] = np.less_equal(self.u.v, self.lower.v)
+
+        if not self.no_upper:
+            self.zu[:] = np.greater_equal(self.u.v, self.upper.v)
+        if not self.no_lower:
+            self.zl[:] = np.less_equal(self.u.v, self.lower.v)
+
         self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
 
 
@@ -202,9 +266,9 @@ class SortedLimiter(Limiter):
             reset_in[highest_n] = 0
             reset_out = 1 - reset_in
 
-            self.zi[:] = np.logical_or(reset_in, self.zi).astype(np.float64)
-            self.zl[:] = np.logical_and(reset_out, self.zl).astype(np.float64)
-            self.zu[:] = np.logical_and(reset_out, self.zu).astype(np.float64)
+            self.zi[:] = np.logical_or(reset_in, self.zi)
+            self.zl[:] = np.logical_and(reset_out, self.zl)
+            self.zu[:] = np.logical_and(reset_out, self.zu)
 
 
 class HardLimiter(Limiter):
@@ -235,8 +299,8 @@ class AntiWindup(Limiter):
         by the anti-windup-limiter.
     """
 
-    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, state=None):
-        super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name)
+    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None, state=None):
+        super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name, info=info)
         self.state = state if state else u
 
     def check_var(self, *args, **kwargs):
@@ -249,40 +313,52 @@ class AntiWindup(Limiter):
         """
         Check the variables and equations and set the limiter flags.
         """
-        self.zu[:] = np.greater(self.u.v, self.upper.v)
-        self.zl[:] = np.less(self.u.v, self.lower.v)
+        self.zu[:] = np.logical_and(np.greater_equal(self.u.v, self.upper.v),
+                                    np.greater_equal(self.state.e, 0))
+        self.zl[:] = np.logical_and(np.less_equal(self.u.v, self.lower.v),
+                                    np.less_equal(self.state.e, 0))
         self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
 
-        self.zu[:] = np.logical_and(self.zu, np.greater(self.state.e, 0)).astype(np.float64)
-        self.zl[:] = np.logical_and(self.zl, np.less(self.state.e, 0)).astype(np.float64)
-        self.zi[:] = np.logical_not(
-            np.logical_or(self.zu.astype(np.bool),
-                          self.zl.astype(np.bool))).astype(np.float64)
-
     def set_eq(self):
         """
         Reset differential equation values based on limiter flags.
+
+        Notes
+        -----
+        The current implementation reallocates memory for `self.x_set` in each call.
+        Consider improving for speed. (TODO)
         """
+
+        # must flush the `x_set` list at the beginning
         self.x_set = list()
+
         if not np.all(self.zi):
-            self.state.e = self.state.e * self.zi
-            self.state.v = self.state.v * self.zi + self.upper.v * self.zu + self.lower.v * self.zl
-            self.x_set.append((self.state.a, self.state.v))
+            idx = np.where(self.zi == 0)
+            self.state.e[:] = self.state.e * self.zi
+            self.state.v[:] = self.state.v * self.zi + self.upper.v * self.zu + self.lower.v * self.zl
+            self.x_set.append((self.state.a[idx], self.state.v[idx]))
+            # logger.debug(f'AntiWindup for states {self.state.a[idx]}')
+
+        # Very important note:
+        # `System.fg_to_dae` is called after `System.l_update_eq`, which calls this function.
+        # Equation values set in `self.state.e` is collected by `System._e_to_dae`, while
+        # variable values are collected by the separate loop in `System.fg_to_dae`.
+        # Also, equation values are processed in `TDS` for resetting the `q`.
 
 
 class Selector(Discrete):
     """
-    Selection of variables using the provided reduce function.
+    Selection between two variables using the provided reduce function.
 
     The reduce function should take the given number of arguments. An example function is `np.maximum.reduce`
     which can be used to select the maximum.
 
-    Names are in `s0`, `s1`, ... and `sn`
+    Names are in `s0`, `s1`.
 
     Warnings
     --------
-    A potential bug when values in different inputs are equal.
-    FIXME: More than one flag will se true in this case.
+    A potential bug when more than two inputs are provided, and values in different inputs are equal.
+    Only two inputs are allowed.
 
     Examples
     --------
@@ -291,7 +367,7 @@ class Selector(Discrete):
     After the definitions of `v0` and `v1`, define the algebraic variable `vmax` for the largest value,
     and a selector `vs` ::
 
-        self.vmax = Algeb(v_str='maximum(v0, v1) - vmax',
+        self.vmax = Algeb(v_str='maximum(v0, v1)',
                           tex_name='v_{max}',
                           e_str='vs_s0 * v0 + vs_s1 * v1 - vmax')
 
@@ -309,18 +385,23 @@ class Selector(Discrete):
     See Also
     --------
     numpy.ufunc.reduce : NumPy reduce function
+
+    andes.core.block.HVGate
+
+    andes.core.block.LVGate
     """
-    def __init__(self, *args, fun, tex_name=None):
-        super().__init__(tex_name=tex_name)
+    def __init__(self, *args, fun, tex_name=None, info=None):
+        super().__init__(tex_name=tex_name, info=info)
+        # TODO: only allow two inputs
         self.input_vars = args
         self.fun = fun
         self.n = len(args)
-        self._s = [0] * self.n
         self._inputs = None
         self._outputs = None
 
+        # TODO: allow custom initial value
         for i in range(len(self.input_vars)):
-            self.__dict__[f's{i}'] = 0
+            self.__dict__[f's{i}'] = np.array([0])
 
         self.export_flags = [f's{i}' for i in range(len(self.input_vars))]
         self.export_flags_tex = [f's_{i}' for i in range(len(self.input_vars))]
@@ -329,10 +410,17 @@ class Selector(Discrete):
         """
         Set the i-th variable's flags to 1 if the return of the reduce function equals the i-th input.
         """
-        self._inputs = [self.input_vars[i].v for i in range(self.n)]
-        self._outputs = self.fun(self._inputs)
+        if self._inputs is None:
+            # input is only evaluated at the first time due to memory stability
+            self._inputs = [self.input_vars[i].v for i in range(self.n)]
+
+        if self._outputs is None:
+            self._outputs = self.fun(self._inputs)
+        else:
+            self._outputs[:] = self.fun(self._inputs)
+
         for i in range(self.n):
-            self.__dict__[f's{i}'][:] = np.equal(self._inputs[i], self._outputs).astype(int)
+            self.__dict__[f's{i}'][:] = np.equal(self._inputs[i], self._outputs)
 
 
 class Switcher(Discrete):
@@ -391,18 +479,16 @@ class Switcher(Discrete):
     def check_var(self, *args, **kwargs):
         """
         Set the switcher flags based on inputs. Uses cached flags if cache is set to True.
-
-        TODO: check if all inputs are valid
         """
         if self.cache and self._eval:
             return
 
         for v in self.u.v:
-            if v not in self.options:
+            if v not in self.options and not np.isnan(v):
                 raise ValueError(f'option {v} is invalid for {self.owner.class_name}.{self.u.name}. '
                                  f'Options are {self.options}.')
         for i in range(len(self.options)):
-            self.__dict__[f's{i}'][:] = np.equal(self.u.v, self.options[i]).astype(np.float64)
+            self.__dict__[f's{i}'][:] = np.equal(self.u.v, self.options[i])
 
         self._eval = True
 
@@ -529,10 +615,6 @@ class DeadBand(Limiter):
         self.zur[:] = np.equal(self.zu + zi, 2) + self.zur * np.equal(zi, self.zi)
         self.zlr[:] = np.equal(self.zl + zi, 2) + self.zlr * np.equal(zi, self.zi)
 
-        self.zu[:] = zu.astype(np.float64)
-        self.zl[:] = zl.astype(np.float64)
-        self.zi[:] = zi.astype(np.float64)
-
 
 class Delay(Discrete):
     """
