@@ -1,7 +1,13 @@
 from typing import Optional, Union, Callable, Type
 from andes.core.param import BaseParam
 from andes.utils.func import list_flatten
+from andes.core.common import dummify
 from andes.shared import np, ndarray
+import logging
+from collections import OrderedDict
+from andes.utils.tab import Tab
+
+logger = logging.getLogger(__name__)
 
 
 class BaseService(object):
@@ -21,11 +27,12 @@ class BaseService(object):
     owner : Model
         The hosting/owner model instance
     """
+
     def __init__(self, name: str = None, tex_name: str = None, info: str = None, vtype: Type = None):
         self.name = name
         self.tex_name = tex_name if tex_name else name
         self.info = info
-        self.vtype = type  # type for `v`. NOT IN USE.
+        self.vtype = vtype  # type for `v`. NOT IN USE.
         self.owner = None
 
     def get_names(self):
@@ -86,10 +93,11 @@ class ConstService(BaseService):
     v : array-like or a scalar
         ConstService value
     """
+
     def __init__(self,
                  v_str: Optional[str] = None,
                  v_numeric: Optional[Callable] = None,
-                 name=None, tex_name=None, info=None):
+                 name: Optional[str] = None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.v_str = v_str
         self.v_numeric = v_numeric
@@ -97,7 +105,69 @@ class ConstService(BaseService):
 
     def assign_memory(self, n):
         """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(self.n)
+        self.v = np.zeros(n)
+
+
+class VarService(ConstService):
+    """
+    Variable service that gets updated in each step/loop as variables change.
+
+    This class is useful when one has non-differentiable algebraic equations,
+    which make use of `abs()`, `re` and `im`.
+    Instead of creating `Algeb`, one can put the equation in `VarService`,
+    which will be updated before solving algebraic equations.
+
+    Examples
+    --------
+    In ESST3A model, the voltage and current sensors (vd + jvq), (Id + jIq)
+    estimate the sensed VE using equation
+
+    .. math ::
+
+        VE = | K_{PC}*(v_d + 1j v_q) + 1j (K_I + K_{PC}*X_L)*(I_d + 1j I_q)|
+
+    One can use `VarService` to implement this equation ::
+
+        self.VE = VarService(tex_name='V_E',
+                             info='VE',
+                             v_str='Abs(KPC*(vd + 1j*vq) + 1j*(KI + KPC*XL)*(Id + 1j*Iq))',
+                             )
+
+    Warnings
+    --------
+    `VarService` is not solved with other algebraic equations, meaning that
+    there is one step "delay" between the algebraic variables and `VarService`.
+    Use an algebraic variable whenever possible.
+    """
+
+    pass
+
+
+class PostInitService(ConstService):
+    """
+    Constant service that gets stored once after init.
+
+    This service is useful when one need to store initialization
+    values stored in variables.
+
+    Examples
+    --------
+    In ESST3A model, the `vf` variable is initialized followed by other
+    variables. One can store the initial `vf` into `vf0` so that equation
+    ``vf - vf0 = 0`` will hold. ::
+
+        self.vref0 = PostInitService(info='Initial reference voltage input',
+                                     tex_name='V_{ref0}',
+                                     v_str='vref',
+                                     )
+
+    Since all `ConstService` are evaluated before equation evaluation,
+    without using PostInitService, one will need to create lots
+    of `ConstService` to store values in the initialization path
+    towards `vf0`, in order to correctly initialize `vf`.
+    """
+
+    pass
 
 
 class ExtService(BaseService):
@@ -129,11 +199,14 @@ class ExtService(BaseService):
                                      tex_name='P_0')
 
     """
+
     def __init__(self,
                  model: str,
                  src: str,
                  indexer: BaseParam,
                  attr='v',
+                 allow_none=False,
+                 default=0,
                  name: str = None,
                  tex_name: str = None,
                  info=None,
@@ -143,11 +216,13 @@ class ExtService(BaseService):
         self.src = src
         self.indexer = indexer
         self.attr = attr
+        self.allow_none = allow_none
+        self.default = default
         self.v = np.array([0.])
 
     def assign_memory(self, n):
         """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(self.n)
+        self.v = np.zeros(n)
 
     def link_external(self, ext_model):
         """
@@ -164,7 +239,10 @@ class ExtService(BaseService):
             return
 
         # the same `get` api for Group and Model
-        self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr=self.attr)
+        self.v = ext_model.get(src=self.src, idx=self.indexer.v, attr=self.attr,
+                               allow_none=self.allow_none,
+                               default=self.default,
+                               )
 
 
 class BackRef(BaseService):
@@ -228,25 +306,35 @@ class BackRef(BaseService):
     andes.core.service.NumReduce : A more complete example using BackRef to build the COI model
 
     """
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.export = False
         self.v = list()
 
 
-class OptionalSelect(BaseService):
+class DataSelect(BaseService):
     """
-    Class for selecting values for optional DataParam.
+    Class for selecting values for optional DataParam or NumParam.
 
     This service is a v-provider that uses optional DataParam if available with a fallback.
 
+    DataParam will be tested for `None`, and NumParam will be tested with `np.isnan()`.
+
     Notes
     -----
-    An use case of OptionalSelect is remote bus. One can do ::
+    An use case of DataSelect is remote bus. One can do ::
 
-        self.buss = OptionalSelect(option=self.busr, fallback=self.bus)
+        self.buss = DataSelect(option=self.busr, fallback=self.bus)
 
     Then, pass ``self.buss`` instead of ``self.bus`` as indexer to retrieve voltages.
+
+    Another use case is to allow an optional turbine rating. One can do ::
+
+        self.Tn = NumParam(default=None)
+        self.Sg = ExtParam(...)
+        self.Sn = DataSelect(Tn, Sg)
+
     """
 
     def __init__(self,
@@ -254,14 +342,21 @@ class OptionalSelect(BaseService):
                  fallback,
                  name: Optional[str] = None,
                  tex_name: Optional[str] = None,
-                 info: Optional[str] = None,):
-        super().__init__(name=name, tex_name=tex_name, info=info,)
+                 info: Optional[str] = None,
+                 ):
+        super().__init__(name=name, tex_name=tex_name, info=info, )
         self.optional = optional
         self.fallback = fallback
+        self._v = None
 
     @property
     def v(self):
-        return [opt if opt is not None else fb for opt, fb in zip(self.optional.v, self.fallback.v)]
+        if self._v is None:
+            self._v = [v1 if v1 is not None and not np.isnan(v1)
+                       else v2
+                       for v1, v2 in zip(self.optional.v, self.fallback.v)]
+
+        return self._v
 
 
 class DeviceFinder(BaseService):
@@ -301,15 +396,23 @@ class DeviceFinder(BaseService):
 
     def find_or_add(self, system):
         mdl = system.models[self.model]
-        found_idx = mdl.find_idx((self.idx_name, ), (self.link.v, ), allow_missing=True)
+        found_idx = mdl.find_idx((self.idx_name,), (self.link.v,),
+                                 allow_none=True, default=None)
 
         action = False
         for ii, idx in enumerate(found_idx):
-            if idx is False:
+            if idx is None:
                 action = True
                 new_idx = system.add(self.model, {self.idx_name: self.link.v[ii]})
                 self.u.v[ii] = new_idx
 
+                logger.info(f"{self.owner.class_name} <{self.owner.idx.v[ii]}> "
+                            f"added {self.model} <{new_idx}> "
+                            f"on {self.idx_name} <{self.link.v[ii]}>")
+            else:
+                action = True
+                self.u.v[ii] = idx
+
         if action:
             mdl.list2array()
             mdl.refresh_inputs()
@@ -333,17 +436,14 @@ class OperationService(BaseService):
 
     IdxRepeat : Service for repeating 1-D IdxParam/ v-list following a sub-pattern
     """
+
     def __init__(self,
-                 ref: BackRef,
-                 u=None,
                  name=None,
                  tex_name=None,
                  info=None,
                  ):
         self._v = None
-        super().__init__(name=name, tex_name=tex_name, info=info,)
-        self.u = u
-        self.ref = ref
+        super().__init__(name=name, tex_name=tex_name, info=info, )
         self.v_str = None
 
     @property
@@ -412,6 +512,7 @@ class NumReduce(OperationService):
     Thus, `self.Vn_mean.v` will become ``[318.33, 220]``.
 
     """
+
     def __init__(self,
                  u,
                  ref: BackRef,
@@ -420,7 +521,9 @@ class NumReduce(OperationService):
                  tex_name=None,
                  info=None,
                  ):
-        super().__init__(u=u, ref=ref, name=name, tex_name=tex_name, info=info)
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.u = u
+        self.ref = ref
         self.fun = fun
 
     @property
@@ -503,11 +606,14 @@ class NumRepeat(OperationService):
     preserved in the sub-equation, the derivatives can be calculated correctly.
 
     """
+
     def __init__(self,
                  u,
                  ref,
                  **kwargs):
-        super().__init__(u=u, ref=ref, **kwargs)
+        super().__init__(**kwargs)
+        self.u = u
+        self.ref = ref
 
     @property
     def v(self):
@@ -537,11 +643,14 @@ class IdxRepeat(OperationService):
     This class has the same functionality as :py:class:`andes.core.service.NumRepeat`
     but only operates on IdxParam, DataParam or NumParam.
     """
+
     def __init__(self,
                  u,
                  ref,
                  **kwargs):
-        super().__init__(u=u, ref=ref, **kwargs)
+        super().__init__(**kwargs)
+        self.u = u
+        self.ref = ref
 
     @property
     def v(self):
@@ -587,14 +696,248 @@ class RefFlatten(OperationService):
                           indexer=self.SynGenIdx, export=False,
                           )
     """
+
     def __init__(self, ref, **kwargs):
-        super().__init__(ref=ref, **kwargs)
+        super().__init__(**kwargs)
+        self.ref = ref
 
     @property
     def v(self):
         return list_flatten(self.ref.v)
 
 
+class NumSelect(OperationService):
+    """
+    Class for selecting values for optional NumParam.
+
+    Notes
+    -----
+    One use case is to allow an optional turbine rating. One can do ::
+
+        self.Tn = NumParam(default=None)
+        self.Sg = ExtParam(...)
+        self.Sn = DataSelect(Tn, Sg)
+
+    """
+
+    def __init__(self,
+                 optional,
+                 fallback,
+                 name: Optional[str] = None,
+                 tex_name: Optional[str] = None,
+                 info: Optional[str] = None,
+                 ):
+        super().__init__(name=name, tex_name=tex_name, info=info)
+        self.optional = optional
+        self.fallback = fallback
+        self._v = None
+
+    @property
+    def v(self):
+        if self._v is None:
+            self._v = [v1 if not np.isnan(v1)
+                       else v2
+                       for v1, v2 in zip(self.optional.v, self.fallback.v)]
+
+            self._v = np.array(self._v)
+
+        return self._v
+
+
+class InitChecker(OperationService):
+    """
+    Class for checking init values against known typical values.
+
+    Instances will be stored in `Model.services_post` and
+    `Model.services_icheck`, which will be checked in
+    `Model.post_init_check()` after initialization.
+
+    Parameters
+    ----------
+    u
+        v-provider to be checked
+    lower : float, BaseParam, BaseVar, BaseService
+        lower bound
+    upper : float, BaseParam, BaseVar, BaseService
+        upper bound
+    equal : float, BaseParam, BaseVar, BaseService
+        values that the value from `v_str` should equal
+    not_equal : float, BaseParam, BaseVar, BaseService
+        values that should not equal
+    enable : bool
+        True to enable checking
+
+    Examples
+    --------
+    Let's say generator excitation voltages are known to be in
+    the range of 1.6 - 3.0 per unit. One can add the following
+    instance to `GENBase` ::
+
+        self._vfc = InitChecker(u=self.vf,
+                                info='vf range',
+                                lower=1.8,
+                                upper=3.0,
+                                )
+
+    `lower` and `upper` can also take v-providers instead of
+    float values.
+
+    One can also pass float values from Config to make it
+    adjustable as in our implementation of ``GENBase._vfc``.
+    """
+
+    def __init__(self, u, lower=None, upper=None, equal=None, not_equal=None,
+                 enable=True, error_out=False, **kwargs):
+        super().__init__(**kwargs)
+        self.u = u
+        self.lower = dummify(lower) if lower is not None else None
+        self.upper = dummify(upper) if upper is not None else None
+        self.equal = dummify(equal) if equal is not None else None
+        self.not_equal = dummify(not_equal) if not_equal is not None else None
+        self.enable = enable
+        self.error_out = error_out
+
+    def check(self):
+        """
+        Check the bounds and equality conditions.
+        """
+        if not self.enable:
+            return
+
+        def _not_all_close(a, b):
+            return np.logical_not(np.isclose(a, b))
+
+        if self._v is None:
+            self._v = np.zeros_like(self.u.v)
+
+        checks = [(self.lower, np.less_equal, "violation of the lower limit", "limit"),
+                  (self.upper, np.greater_equal, "violation of the upper limit", "limit"),
+                  (self.equal, _not_all_close, 'should be equal', "expected"),
+                  (self.not_equal, np.equal, 'should not be equal', "not expected")
+                  ]
+
+        for check in checks:
+            limit = check[0]
+            func = check[1]
+            text = check[2]
+            text2 = check[3]
+            if limit is None:
+                continue
+
+            self.v[:] = np.logical_or(self.v, func(self.u.v, limit.v))
+
+            pos = np.argwhere(func(self.u.v, limit.v)).ravel()
+
+            if len(pos) == 0:
+                continue
+            idx = [self.owner.idx.v[i] for i in pos]
+            lim_v = limit.v * np.ones(self.n)
+
+            title = f'{self.owner.class_name} {self.info} {text}.'
+
+            err_dict = OrderedDict([('idx', idx),
+                                    ('values', self.u.v[pos]),
+                                    (f'{text2}', lim_v[pos]),
+                                    ])
+            data = list(map(list, zip(*err_dict.values())))
+
+            tab = Tab(title=title, data=data, header=list(err_dict.keys()))
+            if self.error_out:
+                logger.error(tab.draw())
+            else:
+                logger.warning(tab.draw())
+
+        self.v[:] = np.logical_not(self.v)
+
+
+class FlagNotNone(BaseService):
+    """
+    Class for flagging non-None indices as 1 and None indices as 0 in a numpy array.
+
+    Warnings
+    --------
+    FlagNotNone can only be applied to BaseParam with `cache=True`.
+    Applying to Service will fail until cache is False (at a performance cost).
+    """
+
+    def __init__(self: BaseParam, indexer, to_flag=None, name=None, tex_name=None, info=None, cache=True):
+        BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.cache = cache
+        self.to_flag = to_flag
+        self.indexer = indexer
+        self._v = None
+
+    @property
+    def v(self):
+        new = False
+        if self._v is None:
+            self._v = np.zeros_like(self.indexer.v, dtype=float)
+            new = True
+
+        if not self.cache or new:
+            self._v[:] = np.array([0 if i == self.to_flag else 1
+                                   for i in self.indexer.v])
+        return self._v
+
+
+class Replace(BaseService):
+    """
+    Replace parameters with new values if the function returns True
+    """
+
+    def __init__(self, old_val, flt, new_val, name=None, tex_name=None, info=None, cache=True):
+        BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.cache = cache
+        self.filter = flt  # function
+        self.old_val = old_val
+        self.new_val = dummify(new_val)
+        self._v = None
+
+    @property
+    def v(self):
+        new = False
+        if self._v is None or not self.cache:
+            self._v = np.zeros_like(self.old_val.v, dtype=float)
+            new = True
+
+        if not self.cache or new:
+            new_v = self.new_val.v * np.ones_like(self.old_val.v)
+            flt = self.filter(self.old_val.v)
+            self._v[:] = new_v * flt + self.old_val.v * (1 - flt)
+
+        return self._v
+
+
+class ParamCalc(BaseService):
+    """
+    Parameter calculation service.
+
+    Useful to create parameters calculated instantly from existing ones.
+    """
+
+    def __init__(self, param1, param2, func, name=None, tex_name=None, info=None,
+                 cache=True):
+        BaseService.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.param1 = param1
+        self.param2 = param2
+        self.func = func
+        self.cache = cache
+        self._v = None
+
+    @property
+    def v(self):
+        new = False
+        if self._v is None:
+            new = True
+            self._v = np.zeros_like(self.param1.v, dtype=float)
+
+        if not self.cache or new:
+            self._v[:] = self.func(self.param1.v,
+                                   self.param2.v)
+
+        return self._v
+
+
 class RandomService(BaseService):
     """
     A service type for generating random numbers.
@@ -611,6 +954,7 @@ class RandomService(BaseService):
     The value will be randomized every time it is accessed. Do not use it if the value needs to be stable for
     each simulation step.
     """
+
     def __init__(self, func=np.random.rand, **kwargs):
         super(RandomService, self).__init__(**kwargs)
         self.func = func
