@@ -1,5 +1,7 @@
 from typing import Optional, Union, List
-from andes.core.param import BaseParam, DummyValue
+from andes.core.param import BaseParam
+from andes.core.common import DummyValue
+from andes.core.discrete import Discrete
 from andes.core.service import BaseService
 from andes.models.group import GroupBase
 from andes.shared import np, ndarray
@@ -7,10 +9,10 @@ from andes.shared import np, ndarray
 
 class BaseVar(object):
     """
-    Base variable class
+    Base variable class.
 
-    This class can be used to instantiate a variable as
-    an attribute of a model class.
+    Derived classes `State` and `Algeb` should be used to build
+    model variables.
 
     Parameters
     ----------
@@ -23,6 +25,9 @@ class BaseVar(object):
     tex_name : str
         LaTeX-formatted variable name. If is None, use `name`
         instead.
+    discrete : Discrete
+        Associated discrete component. Will call `check_var` on
+        the discrete component.
 
     Attributes
     ----------
@@ -43,6 +48,7 @@ class BaseVar(object):
                  v_str: Optional[Union[str, float]] = None,
                  v_iter: Optional[str] = None,
                  e_str: Optional[str] = None,
+                 discrete: Optional[Discrete] = None,
                  v_setter: Optional[bool] = False,
                  e_setter: Optional[bool] = False,
                  addressable: Optional[bool] = True,
@@ -56,33 +62,57 @@ class BaseVar(object):
 
         self.tex_name = tex_name if tex_name else name
         self.owner = None  # instance of the owner Model
-        self.id = None     # variable internal index inside a model (assigned in run time)
-
-        self.n = 0
-        self.a: Optional[Union[ndarray, List]] = np.array([], dtype=int)  # address array
-        self.v: Optional[Union[ndarray, float]] = np.array([], dtype=np.float)  # variable value array
-        self.e: Optional[Union[ndarray, float]] = np.array([], dtype=np.float)   # equation value array
+        self.id = None     # variable internal index inside a model (assigned in run time) FIXME: not in use
 
         self.v_str = v_str  # equation string (v = v_str) for variable initialization
         self.v_iter = v_iter  # the implicit equation (0 = v_iter) for iterative initialization
         self.e_str = e_str  # string for symbolic equation
 
+        self.discrete = discrete
         self.v_setter = v_setter  # True if this variable sets the variable value
         self.e_setter = e_setter  # True if this var sets the equation value
         self.addressable = addressable  # True if this var needs to be assigned an address FIXME: not in use
-        self.export = export  # True if this var's value needs to exported
-        self.diag_eps = diag_eps  # small value to be added to the Jacobian matrix
+        self.export = export            # True if this var's value needs to exported
+        self.diag_eps = diag_eps        # small diagonal value (1e-6) to be added to `dae.gy`
+
+        # attributes assigned by `set_address`
+        self.n = 0
+        self.a: ndarray = np.array([], dtype=int)       # address array
+        self.v: ndarray = np.array([], dtype=np.float)  # variable value array
+        self.e: ndarray = np.array([], dtype=np.float)  # equation value array
+
+        self.av: ndarray = np.array([], dtype=int)      # FIXME: future var. address array
+        self.ae: ndarray = np.array([], dtype=int)      # FIXME: future equation address array
+
+        # internal flags
+        # NOTE:
+        # contiguous is only True for internal variables of models with flag ``collate`` equal to ``False``.
+        self._contiguous = False  # True if if address is contiguous to allow slicing into arrays without copy.
+
+        self.e_inplace = False    # True if `self.e` is in-place access to `System.dae.__dict__[self.e_code]`
+        self.v_inplace = False    # True if `self.v` is in-place access to `System.dae.__dict__[self.v_code]`
+        self.allow_none = False   # True to allow None in address (NOT IN USE)
 
     def reset(self):
-        self.a = np.array([], dtype=int)
-        self.v = np.array([], dtype=np.float)
-        self.e = np.array([], dtype=np.float)
+        """
+        Reset the internal numpy arrays and flags.
+        """
+        self.n = 0
+        self.a[:] = 0
+        self.v[:] = 0
+        self.e[:] = 0
+        self.av[:] = 0
+        self.ae[:] = 0
+
+        self._contiguous = False
+        self.e_inplace = False
+        self.v_inplace = False
 
     def __repr__(self):
         span = []
         if 1 <= self.n <= 20:
             span = self.a.tolist()
-            span = ','.join([str(i) for i in span])
+            span = ', '.join([str(i) for i in span])
         elif self.n > 20:
             if not isinstance(self, ExtVar):
                 span.append(self.a[0])
@@ -95,19 +125,55 @@ class BaseVar(object):
 
         return f'{self.__class__.__name__}, {self.owner.__class__.__name__}.{self.name}{span}'
 
-    def set_address(self, addr):
+    def set_address(self, addr: ndarray, contiguous=False):
         """
-        Set the address of this variables
+        Set the address of internal variables.
 
         Parameters
         ----------
-        addr : array-like
+        addr : ndarray
             The assigned address for this variable
+        contiguous : bool, optional
+            If the addresses are contiguous
         """
         self.a = addr
         self.n = len(self.a)
-        self.v = np.zeros(self.n)
-        self.e = np.zeros(self.n)
+
+        # NOT IN USE
+        self.ae = np.array(self.a)
+        self.av = np.array(self.a)
+        # -----------
+
+        self._contiguous = contiguous
+
+        if self._contiguous:
+            if self.e_setter is False:
+                self.e_inplace = True
+
+            if self.v_setter is False:
+                self.v_inplace = True
+
+    def set_arrays(self, dae):
+        """
+        Set the equation and values arrays.
+
+        It slicing into DAE (when contiguous) or allocating new memory (when not contiguous).
+
+        Parameters
+        ----------
+        dae : DAE
+            Reference to System.dae
+        """
+        slice_idx = slice(self.a[0], self.a[-1] + 1)
+        if self.v_inplace:
+            self.v = dae.__dict__[self.v_code][slice_idx]
+        else:
+            self.v = np.zeros(self.n)
+
+        if self.e_inplace:
+            self.e = dae.__dict__[self.e_code][slice_idx]
+        else:
+            self.e = np.zeros(self.n)
 
     def get_names(self):
         return [self.name]
@@ -136,6 +202,13 @@ class State(BaseVar):
     """
     Differential variable class, an alias of the `BaseVar`.
 
+    Parameters
+    ----------
+    t_const : BaseParam, DummyValue
+        Left-hand time constant for the differential equation.
+        Time constants will not be evaluated as part of the differential equation.
+        They will be collected to array `dae.Tf` to multiply to the right-hand side `dae.f`.
+
     Attributes
     ----------
     e_code : str
@@ -154,6 +227,7 @@ class State(BaseVar):
                  v_str: Optional[Union[str, float]] = None,
                  v_iter: Optional[str] = None,
                  e_str: Optional[str] = None,
+                 discrete: Optional[Discrete] = None,
                  t_const: Optional[Union[BaseParam, DummyValue]] = None,
                  v_setter: Optional[bool] = False,
                  e_setter: Optional[bool] = False,
@@ -168,6 +242,7 @@ class State(BaseVar):
                          v_str=v_str,
                          v_iter=v_iter,
                          e_str=e_str,
+                         discrete=discrete,
                          v_setter=v_setter,
                          e_setter=e_setter,
                          addressable=addressable,
@@ -195,6 +270,8 @@ class ExtVar(BaseVar):
         A parameter of the hosting model, used as indices into
         the source model and variable. If is None, the source
         variable address will be fully copied.
+    allow_none : bool
+        True to allow None in indexer
 
     Attributes
     ----------
@@ -212,6 +289,7 @@ class ExtVar(BaseVar):
                  model: str,
                  src: str,
                  indexer: Optional[Union[List, ndarray, BaseParam, BaseService]] = None,
+                 allow_none: Optional[bool] = False,
                  name: Optional[str] = None,
                  tex_name: Optional[str] = None,
                  info: Optional[str] = None,
@@ -241,6 +319,7 @@ class ExtVar(BaseVar):
         self.model = model
         self.src = src
         self.indexer = indexer
+        self.allow_none = allow_none
         self.parent = None
         self._idx = None
         self._n = []
@@ -264,6 +343,16 @@ class ExtVar(BaseVar):
             idx += n
         return out
 
+    def set_address(self, addr, contiguous=False):
+        """
+        Empty function.
+        """
+        pass
+
+    def set_arrays(self, dae):
+        """Empty function."""
+        pass
+
     def link_external(self, ext_model):
         """
         Update variable addresses provided by external models
@@ -280,7 +369,16 @@ class ExtVar(BaseVar):
         ----------
         ext_model : Model
             Instance of the parent model
+
+        Warnings
+        --------
+        `link_external` does not check if the ExtVar type is the same
+        as the original variable to reduce performance overhead.
+        It will be a silent error (a dimension too small error from `dae.build_pattern`)
+        if a model uses `ExtAlgeb` to access a `State`, or vice versa.
+
         """
+
         self.parent = ext_model
 
         if isinstance(ext_model, GroupBase):
@@ -291,14 +389,21 @@ class ExtVar(BaseVar):
                 self._n = [len(self.indexer.v)]
                 self._idx = self.indexer.v
 
-            self.a = ext_model.get(src=self.src, idx=self._idx, attr='a').astype(int)
+            # use `0` for non-existent addresses (corr. to None in indexer)
+            self.a = ext_model.get(src=self.src,
+                                   idx=self._idx,
+                                   attr='a',
+                                   allow_none=self.allow_none,
+                                   default=0,
+                                   ).astype(int)
             self.n = len(self.a)
-            self.v = np.zeros(self.n)
-            self.e = np.zeros(self.n)
 
         else:
             original_var = ext_model.__dict__[self.src]
 
+            if self.allow_none:
+                raise NotImplementedError(f"{self.name}: allow_none not implemented for Model")
+
             if self.indexer is not None:
                 uid = ext_model.idx2uid(self.indexer.v)
             else:
@@ -312,51 +417,28 @@ class ExtVar(BaseVar):
 
             # set initial v and e values to zero
             self.n = len(self.a)
-            self.v = np.zeros(self.n)
-            self.e = np.zeros(self.n)
+
+        self.v = np.zeros(self.n)
+        self.e = np.zeros(self.n)
 
 
 class ExtState(ExtVar):
+    """
+    External state variable type.
+
+    Warnings
+    --------
+    ``ExtState`` is not allowed to set ``t_const``, as it will conflict with the
+    source ``State`` variable. In fact, one should not set ``e_str`` for ``ExtState``.
+    """
     e_code = 'f'
     v_code = 'x'
-
-    def __init__(self,
-                 model: str,
-                 src: str,
-                 indexer: Optional[Union[List, ndarray, BaseParam, BaseService]] = None,
-                 name: Optional[str] = None,
-                 tex_name: Optional[str] = None,
-                 info: Optional[str] = None,
-                 unit: Optional[str] = None,
-                 v_str: Optional[str] = None,
-                 v_iter: Optional[str] = None,
-                 e_str: Optional[str] = None,
-                 t_const: Optional[Union[BaseParam, DummyValue]] = None,
-                 v_setter: Optional[bool] = False,
-                 e_setter: Optional[bool] = False,
-                 addressable: Optional[bool] = True,
-                 export: Optional[bool] = True,
-                 diag_eps: Optional[float] = 0.0,
-                 ):
-        super().__init__(model=model,
-                         src=src,
-                         indexer=indexer,
-                         name=name,
-                         tex_name=tex_name,
-                         info=info,
-                         unit=unit,
-                         v_str=v_str,
-                         v_iter=v_iter,
-                         e_str=e_str,
-                         v_setter=v_setter,
-                         e_setter=e_setter,
-                         addressable=addressable,
-                         export=export,
-                         diag_eps=diag_eps,
-                         )
-        self.t_const = t_const
+    t_const = None
 
 
 class ExtAlgeb(ExtVar):
+    """
+    External algebraic variable type.
+    """
     e_code = 'g'
     v_code = 'y'
