@@ -10,9 +10,9 @@ non_jit = OrderedDict([
     ('line', ['Line']),
     ('area', ['Area']),
     ('synchronous', ['GENCLS', 'GENROU']),
-    ('governor', ['TG2', 'TGOV1']),
-    ('exciter', ['EXDC2']),
-    ('pss', ['IEEEST']),
+    ('governor', ['TG2', 'TGOV1', 'IEEEG1']),
+    ('exciter', ['EXDC2', 'IEEEX1', 'ESDC2A', 'EXST1', 'ESST3A', 'SEXS']),
+    ('pss', ['IEEEST', 'ST2CUT']),
     ('measurement', ['BusFreq', 'BusROCOF']),
     ('coi', ['COI', ]),
     ('dcbase', ['Node', 'Ground', 'R', 'L', 'C', 'RCp', 'RCs', 'RLs', 'RLCs', 'RLCp']),
