@@ -89,13 +89,11 @@ class DC2Term(ModelData, Model):
     def __init__(self, system, config):
         ModelData.__init__(self)
         self.node1 = IdxParam(default=None,
-                              tex_name='node_1',
                               info='Node 1 index',
                               mandatory=True,
                               model='Node',
                               )
         self.node2 = IdxParam(default=None,
-                              tex_name='node_2',
                               info='Node 2 index',
                               mandatory=True,
                               model='Node',
@@ -141,13 +139,11 @@ class ACDC2Term(ModelData, Model):
                             mandatory=True,
                             )
         self.node1 = IdxParam(default=None,
-                              tex_name='node_1',
                               info='Node 1 index',
                               mandatory=True,
                               model='Node',
                               )
         self.node2 = IdxParam(default=None,
-                              tex_name='node_2',
                               info='Node 2 index',
                               mandatory=True,
                               model='Node',
@@ -207,7 +203,6 @@ class Ground(ModelData, Model):
     def __init__(self, system, config):
         ModelData.__init__(self)
         self.node = IdxParam(default=None,
-                             tex_name='node',
                              info='Node index',
                              mandatory=True,
                              model='Node',
@@ -240,7 +235,7 @@ class R(DC2Term):
     """
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
         self.R = NumParam(unit='p.u.',
                           info='DC line resistance',
@@ -264,7 +259,7 @@ class L(DC2Term):
     """
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.L = NumParam(unit='p.u.',
@@ -289,7 +284,7 @@ class C(DC2Term):
     """
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.C = NumParam(unit='p.u.',
@@ -319,16 +314,18 @@ class C(DC2Term):
 class RLs(DC2Term):
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.R = NumParam(unit='p.u.',
+                          tex_name='R',
                           info='DC line resistance',
                           non_zero=True,
                           default=0.01,
                           r=True,
                           )
         self.L = NumParam(unit='p.u.',
+                          tex_name='L',
                           info='DC line inductance',
                           non_zero=True,
                           default=0.001,
@@ -353,16 +350,18 @@ class RLs(DC2Term):
 class RCp(DC2Term):
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.R = NumParam(unit='p.u.',
+                          tex_name='R',
                           info='DC line resistance',
                           non_zero=True,
                           default=0.01,
                           r=True,
                           )
         self.C = NumParam(unit='p.u.',
+                          tex_name='C',
                           info='DC capacitance',
                           non_zero=True,
                           default=0.001,
@@ -389,22 +388,25 @@ class RCp(DC2Term):
 class RLCp(DC2Term):
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.R = NumParam(unit='p.u.',
+                          tex_name='R',
                           info='DC line resistance',
                           non_zero=True,
                           default=0.01,
                           r=True,
                           )
         self.L = NumParam(unit='p.u.',
+                          tex_name='L',
                           info='DC line inductance',
                           non_zero=True,
                           default=0.001,
                           r=True,
                           )
         self.C = NumParam(unit='p.u.',
+                          tex_name='C',
                           info='DC capacitance',
                           non_zero=True,
                           default=0.001,
@@ -437,16 +439,18 @@ class RLCp(DC2Term):
 class RCs(DC2Term):
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.R = NumParam(unit='p.u.',
+                          tex_name='R',
                           info='DC line resistance',
                           non_zero=True,
                           default=0.01,
                           r=True,
                           )
         self.C = NumParam(unit='p.u.',
+                          tex_name='C',
                           info='DC capacitance',
                           non_zero=True,
                           default=0.001,
@@ -473,22 +477,25 @@ class RCs(DC2Term):
 class RLCs(DC2Term):
     def __init__(self, system, config):
         DC2Term.__init__(self, system, config)
-        self.flags['pflow'] = True
+        self.flags.pflow = True
         self.group = 'DCLink'
 
         self.R = NumParam(unit='p.u.',
+                          tex_name='R',
                           info='DC line resistance',
                           non_zero=True,
                           default=0.01,
                           r=True,
                           )
         self.L = NumParam(unit='p.u.',
+                          tex_name='L',
                           info='DC line inductance',
                           non_zero=True,
                           default=0.001,
                           r=True,
                           )
         self.C = NumParam(unit='p.u.',
+                          tex_name='C',
                           info='DC capacitance',
                           non_zero=True,
                           default=0.001,
