@@ -29,7 +29,7 @@ class LineData(ModelData):
 
         self.trans = NumParam(default=0, info="transformer branch flag")
         self.tap = NumParam(default=1.0, info="transformer branch tap ratio", tex_name='t_{ap}')
-        self.phi = NumParam(default=0, info="transformer branch phase shift in rad", tex_name=r'\phi')
+        self.phi = NumParam(default=0.0, info="transformer branch phase shift in rad", tex_name=r'\phi')
 
         self.owner = IdxParam(model='Owner', info="owner code")
 
@@ -42,7 +42,7 @@ class Line(LineData, Model):
         LineData.__init__(self)
         Model.__init__(self, system, config)
         self.group = 'ACLine'
-        self.flags['pflow'] = True
+        self.flags.pflow = True
 
         self.a1 = ExtAlgeb(model='Bus', src='a', indexer=self.bus1, tex_name='a_1',
                            info='phase angle of the from bus')
