@@ -1,11 +1,13 @@
 from andes.core.solver import Solver
-from andes.core.config import Config
+from andes.core import Config
 from collections import OrderedDict
 
 
 class BaseRoutine(object):
     """
-    Base routine class to provide instances for system, config, and solver.
+    Base routine class.
+
+    Provides references to system, config, and solver.
     """
 
     def __init__(self, system=None, config=None):
@@ -16,9 +18,15 @@ class BaseRoutine(object):
             self.config.load(config)
 
         self.config.add(OrderedDict((('sparselib', 'klu'),
+                                     ('linsolve', 0),
                                      )))
-        self.config.add_extra("_help", sparselib="linear sparse solver name")
-        self.config.add_extra("_alt", sparselib=("klu", "umfpack"))
+        self.config.add_extra("_help",
+                              sparselib="linear sparse solver name",
+                              linsolve="solve symbolic factorization each step (enable when KLU segfaults)",
+                              )
+        self.config.add_extra("_alt", sparselib=("klu", "umfpack", "spsolve", "cupy"),
+                              linsolve=(0, 1),
+                              )
 
         self.solver = Solver(sparselib=self.config.sparselib)
 
@@ -26,7 +34,7 @@ class BaseRoutine(object):
     def class_name(self):
         return self.__class__.__name__
 
-    def doc(self, max_width=80, export='plain'):
+    def doc(self, max_width=78, export='plain'):
         return self.config.doc(max_width, export)
 
     def init(self):
